@@ -753,6 +753,22 @@ def get_run_by_id(id, session=None):
     return run
 
 
+def get_runs_by_ids(ids, session=None):
+    """Get a list of runs by their ids.
+
+    :param list id: A list of run ids for the run
+    :param session: Optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return: The list of the specified run objects
+    :rtype: list
+    """
+    session = session or get_session()
+    run = db_utils.model_query(models.Run, session).filter(
+        models.Run.id.in_(ids)).all()
+    return run
+
+
 def get_test_run_by_id(test_run_id, session=None):
     """Get an individual test run by it's id.
 
@@ -788,7 +804,8 @@ def get_test_runs_by_test_id(test_id, session=None):
 
 
 def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
-                                  session=None, key=None, value=None):
+                                  session=None, key=None, value=None,
+                                  most_recent_first=False):
     """Get all test runs for a specific test by the test'stest_id column
 
     :param str test_id: The test's test_id (the test_id column in the test
@@ -805,6 +822,8 @@ def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
     :param str value: An optional value for run metadata to filter the test
                       runs on. Must be specified with a key otherwise it does
                       nothing.
+    :param bool most_recent_first: If true order the results list by date of
+                                   test_run start_time in descending order.
 
     :return list: The list of test run objects for the specified test
     :rtype: subunit2sql.models.TestRun
@@ -827,6 +846,9 @@ def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
             models.TestRun.run_id == models.RunMetadata.run_id).filter(
                 models.RunMetadata.key == key,
                 models.RunMetadata.value == value)
+    if most_recent_first:
+        test_runs_query = test_runs_query.order_by(
+            models.TestRun.start_time.desc())
     test_runs = test_runs_query.all()
     return test_runs
 
@@ -1101,7 +1123,7 @@ def get_recent_runs_by_key_value_metadata(key, value, num_runs=10,
                                 date will be returned.
 
     :return list: A list of run objects for the most recent runs.
-    :rtype subunit2sql.db.models.Run
+    :rtype: subunit2sql.db.models.Run
     """
     session = session or get_session()
     results = db_utils.model_query(models.Run, session).join(
@@ -1571,6 +1593,28 @@ def get_recent_failed_runs_by_run_metadata(key, value, num_runs=10,
         models.Run.run_at.desc()).limit(num_runs).all()
 
 
+def get_runs_counts_by_run_metadata(key, value, start_date=None, session=None):
+    """Check runs for a given run metadata pair
+
+    :param str key: The run_metadata key to check runs
+    :param str value: The run_metadata value to check runs
+    :param datetime start_date: The optional starting dates to get runs from.
+                                Nothing older than this date will be returned
+    :param session: Optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return count: A count for a given run metadata key-value pair
+    :rtype: int
+    """
+    session = session or get_session()
+    query = db_utils.model_query(models.Run, session).join(
+        models.RunMetadata, models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key,
+            models.RunMetadata.value == value)
+    query = _filter_runs_by_date(query, start_date)
+    return query.count()
+
+
 def get_runs_by_status_grouped_by_run_metadata(key, start_date=None,
                                                stop_date=None, session=None):
     session = session or get_session()
@@ -1711,3 +1755,29 @@ def get_all_test_run_metadata_keys(session=None):
     session = session or get_session()
     keys = session.query(models.TestRunMetadata.key).distinct().all()
     return [key[0] for key in keys]
+
+
+def get_recent_successful_runs_by_run_metadata(key, value, num_runs=10,
+                                               start_date=None, session=None):
+    """Get a list of recent successful runs for a given run metadata pair
+
+    :param str key: The run_metadata key to get successful runs
+    :param str value: The run_metadata value to get successful runs
+    :param int num_runs: The number of results to fetch, defaults to 10
+    :param datetime start_date: The optional starting dates to get runs from.
+                                Nothing older than this date will be returned
+    :param session: Optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return list: The list of recent failed Run objects
+    :rtype: subunit2sql.models.Run
+    """
+    session = session or get_session()
+    query = db_utils.model_query(models.Run, session).join(
+        models.RunMetadata, models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key,
+            models.RunMetadata.value == value)
+    query = _filter_runs_by_date(query, start_date)
+    return query.filter(models.Run.passes >= 1).filter(
+        models.Run.fails == 0).order_by(
+            models.Run.run_at.desc()).limit(num_runs).all()
